# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class BaseStorage(object):
    """
    基础存储类
    """
    def get(self, key, default=None):
        raise NotImplementedError()
        # 要求子类必须实现的方法
        # Python编程中raise可以实现报出错误的功能，而报错的条件可以由程序员自己去定制。在面向对象编程中，可以先预留一个方法接口不实现，在其子类中实现。
        # 如果要求其子类一定要实现，不实现的时候会导致问题，那么采用raise的方式就很好。
        # 而此时产生的问题分类是NotImplementedError。
    def set(self, key, value, ttl=None):
        raise NotImplementedError()

    def delete(self, key):
        raise NotImplementedError()

    def __getitem__(self, key):
        self.get(key)

    def __setitem__(self, key, value):
        self.set(key, value)

    def __delitem__(self, key):
        self.delete(key)



# -*- coding: utf-8 -*-
from sqlalchemy import String, Integer, DateTime, Column, ForeignKey

from aishowapp.ext import db
from aishowapp.models import BaseDef

user_role = db.Table('user_role',
    db.Column('user_id', db.Integer, db.ForeignKey('users.id')),
    db.Column('role_id', db.Integer, db.ForeignKey('roles.id'))
)
role_permission = db.Table('role_permission',
    db.Column('permission_id', db.Integer, db.ForeignKey('permissions.id')),
    db.Column('role_id', db.Integer, db.ForeignKey('roles.id'))
)

class User(db.Model,BaseDef):

    __tablename__ = 'users'
    id = db.Column(db.Integer, primary_key=True,autoincrement=True)
    username = db.Column(String(255))
    password = db.Column(String(255))
    token = db.Column(String(255))
    likes = db.Column(String(255))
    avatar = db.Column(String(255))
    introduction = db.Column(String(255))
    sex = db.Column(String(255))
    roles = db.relationship('Role', secondary=user_role, backref=db.backref('users', lazy='dynamic'))
    usersearch_id = db.relationship("UserSearch", backref='user')

class UserSearch(db.Model,BaseDef):

    __tablename__ = 'usersearch'

    id = db.Column(db.Integer, primary_key=True,)
    count = db.Column(Integer)
    time = db.Column(DateTime)
    conditions = db.Column(String(255))
    user_id = Column(ForeignKey('users.id'))

class Role(db.Model,BaseDef):

    __tablename__ = 'roles'
    id = db.Column(db.Integer, primary_key=True,autoincrement=True)
    r_name=db.Column(db.String(255))
    permissions = db.relationship('Permission', secondary=role_permission, backref=db.backref('roles', lazy='dynamic'))

class Permission(db.Model,BaseDef):

    __tablename__ = 'permissions'
    id = db.Column(db.Integer, primary_key=True,autoincrement=True)
    p_name=db.Column(db.String(255))
    p_url=db.Column(db.String(255))


#!/usr/bin/env python
import os
COV = None
if os.environ.get('FLASK_COVERAGE'):
    #测试模块
    import coverage
    COV = coverage.coverage(branch=True, include='app/*')
    COV.start()

if os.path.exists('.env'):
    print('Importing environment from .env...')
    for line in open('.env'):
        var = line.strip().split('=')
        if len(var) == 2:
            os.environ[var[0]] = var[1]

from aishow_model_app import create_app
from aishow_model_app.apis import *
from aishow_model_app.models import *

from flask_script import Manager, Shell
from flask_migrate import Migrate, MigrateCommand

app = create_app(os.getenv('FLASK_CONFIG') or 'default')
# manager = Manager(app)
# migrate = Migrate(app, db)


@app.shell_context_processor
def make_shell_context():
    return dict(app=app, db=db)

# manager.add_command("shell", Shell(make_context=make_shell_context))
# manager.add_command('db', MigrateCommand)

#
# @manager.command
# def test(coverage=False):
#     """Run the unit tests."""
#     if coverage and not os.environ.get('FLASK_COVERAGE'):
#         import sys
#         os.environ['FLASK_COVERAGE'] = '1'
#         os.execvp(sys.executable, [sys.executable] + sys.argv)
#     import unittest
#     tests = unittest.TestLoader().discover('tests')
#     unittest.TextTestRunner(verbosity=2).run(tests)
#     if COV:
#         COV.stop()
#         COV.save()
#         print('Coverage Summary:')
#         COV.report()
#         basedir = os.path.abspath(os.path.dirname(__file__))
#         covdir = os.path.join(basedir, 'tmp/coverage')
#         COV.html_report(directory=covdir)
#         print('HTML version: file://%s/index.html' % covdir)
#         COV.erase()


# @manager.command
# def profile(length=25, profile_dir=None):
#     """Start the application under the code profiler."""
#     from werkzeug.contrib.profiler import ProfilerMiddleware
#     app.wsgi_app = ProfilerMiddleware(app.wsgi_app, restrictions=[length],
#                                       profile_dir=profile_dir)
#     app.run()


# @manager.command
# def deploy():
#     """Run deployment tasks."""
#     from flask_migrate import upgrade
#     from app.models import Role, User
#
#     # migrate database to latest revision
#     # upgrade()
#
#     # create user roles
#     Role.insert_roles()
#
#     # create self-follows for all users
#     User.add_self_follows()


if __name__ == '__main__':
    # manager.run()
    app.run(host='0.0.0.0',port=8001)


# -*- coding: utf-8 -*-
from datetime import datetime as cdatetime  # 有时候会返回datatime类型
from datetime import date, time
from flask_sqlalchemy import Model
from sqlalchemy import DateTime, Numeric, Time, Date
from aishow_model_app.ext import db

class BaseDef(object):

    @classmethod
    def queryToDict(cls, models):
        if (isinstance(models, list)):
            if (isinstance(models[0], Model)):
                lst = []
                for model in models:
                    gen = cls.model_to_dict(model)
                    # print('gen',gen)
                    dit = dict((g[0], g[1]) for g in gen)
                    lst.append(dit)
                return lst
            else:
                res = cls.result_to_dict(models)
                return res
        else:
            if (isinstance(models, Model)):
                gen = cls.model_to_dict(models)
                dit = dict((g[0], g[1]) for g in gen)
                return dit
            else:
                res = dict(zip(models.keys(), models))
                cls.find_datetime(res)
                return res

    # 当结果为result对象列表时，result有key()方法
    @classmethod
    def result_to_dict(cls, results):
        res = [dict(zip(r.keys(), r)) for r in results]
        # 这里r为一个字典，对象传递直接改变字典属性
        for r in res:
            cls.find_datetime(r)
        return res

    @classmethod
    def model_to_dict(cls, model):  # 这段来自于参考资源
        for col in model.__table__.columns:
            # print('model_to_dict',)
            # print(col)
            if isinstance(col.type, DateTime):
                value = cls.convert_datetime(getattr(model, col.name))
            elif isinstance(col.type, Numeric):
                if getattr(model, col.name):
                    # print(getattr(model, col.name))
                    # print(type(getattr(model, col.name)))
                    value = float(getattr(model, col.name))
            else:
                value = getattr(model, col.name)
            yield (col.name, value)

    @classmethod
    def find_datetime(cls, value):
        for v in value:
            if (isinstance(value[v], cdatetime)):
                value[v] = cls.convert_datetime(value[v])  # 这里原理类似，修改的字典对象，不用返回即可修改

    @classmethod
    def convert_datetime(cls, value):
        if value:
            if (isinstance(value, (cdatetime, DateTime))):
                return value.strftime("%Y-%m-%d %H:%M:%S")
            elif (isinstance(value, (date, Date))):
                return value.strftime("%Y-%m-%d")
            elif (isinstance(value, (Time, time))):
                return value.strftime("%H:%M:%S")
        else:
            return ""

    def save(self,obj):
        try:
            db.session.add(obj)
            db.session.flush()
            db.session.commit()
        except Exception as e:
            print('save fail',e)


    # def delete(self,obj):
    #     try:
    #         db.session.delete(obj)
    #         db.session.commit()
    #     except Exception as e:
    #         print('save fail',e)

from . import resource_model


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import requests




class BaseAPI(object):

    API_BASE_URL = None

    def __init__(self, client=None):
        self._client = client

    def _get(self, url, params=None, **kwargs):
        from aishow_model_app.baseclient import BaseClient
        bsc = BaseClient()
        self._client = bsc
        if self.API_BASE_URL:
            kwargs['api_base_url'] = self.API_BASE_URL
        # return self._client.get(url, params, **kwargs)
        # return self._client.get(url=none, params={'select_type':select_type,'page':page,'limit':limit}, **kwargs)
        return self._client.get(url, params, **kwargs)

    def _post(self, url, data=None, params=None, **kwargs):
        from aishow_model_app.baseclient import BaseClient
        bsc = BaseClient()
        self._client = bsc
        if self.API_BASE_URL:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.post(url, data, params, **kwargs)



# -*- coding: utf-8 -*-
import json

from flask import jsonify
from flask_restful import Api, Resource, reqparse


from aishow_model_app.models.resource_model import DouyinSpecialLive, ResourceTable, DouyinSingleChainLive, DouyinViewExport, \
    KuaiShouLive, RedbookImageTextLink, TaobaoLive, QitengTaobaoExportLiveOffer, QitengRedbookPrice, \
    QitengDouyinViewPrice


from aishow_model_app.apis.base import BaseAPI
from aishow_model_app.ext import db

ResourceApi = Api(prefix='/platformresource/list')
#达人的增删改查
class TotalResource(Resource,BaseAPI):

    def __init__(self):
        self.parse = reqparse.RequestParser()
        self.parse.add_argument('page', type=int)
        self.parse.add_argument('limit', type=int)
        self.parse.add_argument('user_id', type=str)
        self.parse.add_argument('flag', type=str)
        self.parse.add_argument('select_type', type=int)
        self.parse.add_argument('kol_id', type=str)
        self.parse.add_argument('kol_name', type=str)
        self.parse.add_argument('platform', type=str)
        self.parse.add_argument('type_datil', type=str)
        self.parse.add_argument('company', type=str)
        self.parse.add_argument('contact_name', type=str)
        self.parse.add_argument('contact_phone', type=str)
        self.parse.add_argument('fans', type=float)
        self.parse.add_argument('status', type=str)

    def get(self):
        args = self.parse.parse_args()
        print('resourcetable args get',args)
        print('resourcetable args get',args['select_type'])
        print('resourcetable args get',type(args['select_type']))
        all_list = []
        totle = 0
        #抖音专场直播
        if args['select_type'] == 1:
            dysl_query = DouyinSpecialLive.query.offset((args['page'] - 1) * args['limit']).limit(args['limit']).all()
            totle = DouyinSpecialLive.query.count()
            for item in dysl_query:
                dict = {}
                dict['dysl'] = DouyinSpecialLive.queryToDict(item)
                res_query = item.resource_tables
                if res_query:
                    res_dict = ResourceTable.queryToDict(res_query)
                    dict['rsst'] = res_dict
                all_list.append(dict)
        #抖音单链接
        elif args['select_type'] == 2:
            dyscl_query = DouyinSingleChainLive.query.offset((args['page'] - 1) * args['limit']).limit(args['limit']).all()
            totle = DouyinSingleChainLive.query.count()
            for item in dyscl_query:
                dict = {}
                dict['dyscl'] = DouyinSingleChainLive.queryToDict(item)
                res_query = item.resource_tables
                if res_query:
                    res_dict = ResourceTable.queryToDict(res_query)
                    dict['rsst'] = res_dict
                all_list.append(dict)
        #抖音短视频
        elif args['select_type'] == 3:
            dyscl_query = DouyinViewExport.query.offset((args['page'] - 1) * args['limit']).limit(
                args['limit']).all()
            totle = DouyinViewExport.query.count()
            for item in dyscl_query:
                dict = {}
                dict['dyve'] = DouyinViewExport.queryToDict(item)
                res_query = item.resource_tables
                if res_query:
                    res_dict = ResourceTable.queryToDict(res_query)
                    dict['rsst'] = res_dict
                dysl2_query = QitengDouyinViewPrice.query.filter(QitengDouyinViewPrice.douyin_view_export_id == item.id).first()
                if dysl2_query:
                    dysl_dict = QitengDouyinViewPrice.queryToDict(dysl2_query)
                    for k,v in dysl_dict.items():
                        dict['dyve'][k] = v
                all_list.append(dict)
        #快手直播
        elif args['select_type'] == 4:
            dyscl_query = KuaiShouLive.query.offset((args['page'] - 1) * args['limit']).limit(
                args['limit']).all()
            totle = KuaiShouLive.query.count()
            for item in dyscl_query:
                dict = {}
                dict['ksl'] = KuaiShouLive.queryToDict(item)
                res_query = item.resource_tables
                if res_query:
                    res_dict = ResourceTable.queryToDict(res_query)
                    dict['rsst'] = res_dict
                all_list.append(dict)
        #小红书图文链接
        elif args['select_type'] == 5:
            dyscl_query = RedbookImageTextLink.query.offset((args['page'] - 1) * args['limit']).limit(
                args['limit']).all()
            totle = RedbookImageTextLink.query.count()
            for item in dyscl_query:
                dict = {}
                dict['rbitl'] = RedbookImageTextLink.queryToDict(item)
                res_query = item.resource_tables
                if res_query:
                    res_dict = ResourceTable.queryToDict(res_query)
                    dict['rsst'] = res_dict
                dysl2_query = QitengRedbookPrice.query.filter(
                    QitengRedbookPrice.redbook_image_text_link_id ==  item.id).first()
                if dysl2_query:
                    dysl_dict = QitengDouyinViewPrice.queryToDict(dysl2_query)
                    for k,v in dysl_dict.items():
                        dict['rbitl'][k] = v
                all_list.append(dict)
        #淘宝直播
        elif args['select_type'] == 6:
            dyscl_query = TaobaoLive.query.offset((args['page'] - 1) * args['limit']).limit(
                args['limit']).all()
            totle = TaobaoLive.query.count()
            for item in dyscl_query:
                dict = {}
                dict['tbl'] = TaobaoLive.queryToDict(item)
                res_query = item.resource_tables
                if res_query:
                    res_dict = ResourceTable.queryToDict(res_query)
                    print('res_dict',res_dict)
                    dict['rsst'] = res_dict
                dysl2_query = QitengTaobaoExportLiveOffer.query.filter(
                    QitengTaobaoExportLiveOffer.taobao_live_id == item.id).first()
                if dysl2_query:
                    dysl_dict = QitengTaobaoExportLiveOffer.queryToDict(dysl2_query)
                    print(dysl_dict)
                    for k,v in dysl_dict.items():
                        dict['tbl'][k] = v
                all_list.append(dict)
        # print('all_list',all_list)
        else:
            dict = {}
            rsst_query = ResourceTable.query.all()
            if rsst_query:
                res_dict = ResourceTable.queryToDict(rsst_query)
                # print('res_dict', res_dict)
                # dict['rsst'] = res_dict
                all_list=res_dict
        print('all_list',json.dumps({'code': 20000, 'data': {'total': totle, 'items': all_list }},ensure_ascii=False))
        return jsonify({'code': 20000, 'data': {'total': totle, 'items': all_list }})

    def post(self):
        print('resource post')
        args = self.parse.parse_args()
        res = self.add_updata(args)
        if res:
            return jsonify({'code': 20000, 'data': {'total': 0, 'items': [],'msg':'失败，请输入正确的数据'}})
        else:
        # res_query = ResourceTable.query.offset((int(args['page']) - 1) * int(args['limit'])).limit(
        #     int(args['limit'])).all()
        # totle = ResourceTable.query.count()
        # res_dict = ResourceTable.queryToDict(res_query)
        #
            return self.get()

    def put(self):
        print('resource put')
        args = self.parse.parse_args()
        res = self.add_updata(args)
        if res:
            return jsonify({'code': 20000, 'data': {'total': 0, 'items': [],'msg':'更新失败，请输入正确的数据'}})
        else:
            return self.get()

    def delete(self):
        args = self.parse.parse_args()
        print('resourcetable delete args',args)
        res_obj = ResourceTable.query.filter(ResourceTable.id == int(args['user_id'])).first()
        res_id = res_obj.id
        print('res_id',res_id)
        flag = args.get('flag')
        if flag == '抖音专场直播':  # 抖音专场直播
            print('抖音专场直播')
            douyinsl_obj = DouyinSpecialLive.query.filter(DouyinSpecialLive.resource_table_id == res_id).first()
            if douyinsl_obj:
                try:
                    db.session.delete(douyinsl_obj)
                    db.session.commit()
                except Exception as e:
                    print('e', e)
        elif flag == '抖音单链直播':  # 抖音专场直播
            print('抖音单链直播')
            douyinscl_obj = DouyinSingleChainLive.query.filter(DouyinSingleChainLive.resource_table_id == res_id).first()
            if douyinscl_obj:
                try:
                    db.session.delete(douyinscl_obj)
                    db.session.commit()
                except Exception as e:
                    print('e', e)
        elif flag == '抖音短视频':  # 抖音专场直播
            print('抖音短视频')
            douyinve_obj = DouyinViewExport.query.filter(DouyinViewExport.resource_table_id == res_id).first()
            if douyinve_obj:
                try:
                    db.session.delete(douyinve_obj)
                    db.session.commit()
                except Exception as e:
                    print('e', e)
        elif flag == '快手直播':  # 抖音专场直播
            print('快手直播')
            ks_obj = KuaiShouLive.query.filter(KuaiShouLive.resource_table_id == res_id).first()
            if ks_obj:
                try:
                    db.session.delete(ks_obj)
                    db.session.commit()
                except Exception as e:
                    print('e', e)
        elif flag == '小红书图文链接':  # 抖音专场直播
            print('小红书图文链接')
            rebbookitl_obj = RedbookImageTextLink.query.filter(RedbookImageTextLink.resource_table_id == res_id).first()
            if rebbookitl_obj:
                try:
                    db.session.delete(rebbookitl_obj)
                    db.session.commit()
                except Exception as e:
                    print('e', e)
        elif flag == '淘宝直播':  # 抖音专场直播
            print('淘宝直播')
            tbl_obj = TaobaoLive.query.filter(TaobaoLive.resource_table_id == res_id).first()
            if tbl_obj:
                try:
                    db.session.delete(tbl_obj)
                    db.session.commit()
                except Exception as e:
                    print('e', e)
        if res_obj:
            try:
                db.session.delete(res_obj)
                db.session.commit()
            except Exception as e:
                print('e', e)
        res_query = ResourceTable.query.offset((int(args['page']) - 1) * int(args['limit'])).limit(
            int(args['limit'])).all()
        totle = ResourceTable.query.count()
        res_dict = ResourceTable.queryToDict(res_query)
        return jsonify({'code': 20000, 'data': {'total': totle, 'items': res_dict}})

    def add_updata(self,args):
        status =args['status']
        if status=='create':
            rsourcetab = ResourceTable()
        else :
            rsourcetab = ResourceTable.query.filter(ResourceTable.kol_id==args['kol_id']).first()
        rsourcetab.kol_id = args.get('kol_id')
        rsourcetab.kol_name = args.get('kol_name')
        rsourcetab.platform = args.get('platform')
        rsourcetab.avatar = args.get('avatar')
        rsourcetab.type_datil = args.get('type_datil')
        rsourcetab.company = args.get('company')
        rsourcetab.contact_name = args.get('contact_name')
        rsourcetab.fans = args.get('fans')
        try:
            try:
                rsourcetab.save(rsourcetab)
            except Exception as e:
                print('TotalResource post except', e)
            resource_obj = ResourceTable.query.filter(ResourceTable.kol_id == args.get('kol_id')).all()[0]
            resource_id = resource_obj.id
            print(resource_id)
            flag = args.get('flag')
            if flag == '抖音专场直播':  # 抖音专场直播
                print('抖音专场直播')
                self.parse.add_argument('export_tag', type=str)
                self.parse.add_argument('special_offer', type=int)
                self.parse.add_argument('export_city', type=str)
                self.parse.add_argument('cooperation_case', type=str)
                self.parse.add_argument('douyin_special_cost_price', type=int)
                args1 = self.parse.parse_args()
                print('抖音专场直播 args1', args1)
                douyinsl = DouyinSpecialLive()
                douyinsl.resource_table_id = resource_id
                douyinsl.export_tag = args1.get('export_tag')
                douyinsl.special_offer = args1.get('special_offer')
                douyinsl.export_city = args1.get('export_city')
                douyinsl.cooperation_case = args1.get('cooperation_case')
                douyinsl.douyin_special_cost_price = args1.get('douyin_special_cost_price')
                try:
                    douyinsl.save(douyinsl)
                    # dysl_query = DouyinSpecialLive.query.offset((args1['page']) - 1 * args1['limit']).limit(args1['limit']).all()
                    # print('ok7')
                    # totle = DouyinSpecialLive.query.count()
                    # dysl_dict = DouyinSpecialLive.queryToDict(dysl_query)
                    # return jsonify({'code': 20000, 'data': {'total': totle, 'items': dysl_dict}})
                except Exception as e:
                    print('DouyinSpecialLive save except', e)
            elif flag == '抖音单链直播':  # 抖音单链直播
                self.parse.add_argument('douyin_export_classification', type=str)
                self.parse.add_argument('Single_chain_offer', type=int)
                self.parse.add_argument('introduction', type=str)
                self.parse.add_argument('live_time', type=str)
                self.parse.add_argument('selection_requirements', type=str)
                self.parse.add_argument('remarks', type=str)
                self.parse.add_argument('douyin_single_cost_price', type=int)
                args = self.parse.parse_args()
                print('抖音单链直播',args)
                douyinscl = DouyinSingleChainLive()
                douyinscl.resource_table_id = resource_id
                douyinscl.douyin_export_classification = args.get('douyin_export_classification')
                douyinscl.Single_chain_offer = args.get('Single_chain_offer')
                douyinscl.introduction = args.get('introduction')
                douyinscl.live_time = args.get('live_time')
                douyinscl.selection_requirements = args.get('selection_requirements')
                douyinscl.remarks = args.get('remarks')
                douyinscl.douyin_single_cost_price = args.get('douyin_single_cost_price')
                try:
                    douyinscl.save(douyinscl)
                    dyscl_query = DouyinSpecialLive.query.offset((args['page']) - 1 * args['limit']).limit(
                        args['limit']).all()
                    # totle = DouyinSingleChainLive.query.count()
                    # dyscl_dict = DouyinSingleChainLive.queryToDict(dyscl_query)
                    # return jsonify({'code': 20000, 'data': {'total': totle, 'items': dyscl_dict}})
                except Exception as e:
                    print('DouyinSpecialLive save except', e)
            elif flag == '抖音短视频':  # 抖音短视频
                self.parse.add_argument('export_tag', type=str)
                self.parse.add_argument('introduction', type=str)
                self.parse.add_argument('douyin_home_page', type=str)
                self.parse.add_argument('export_city', type=str)
                self.parse.add_argument('cooperation_case', type=str)
                self.parse.add_argument('better_sell_goods', type=str)
                self.parse.add_argument('douyin_export_classification', type=str)
                self.parse.add_argument('cooperation_mode', type=str)
                self.parse.add_argument('offer_less', type=int)
                self.parse.add_argument('offer_more', type=int)
                self.parse.add_argument('offer_less', type=int)
                self.parse.add_argument('star_offer', type=int)
                self.parse.add_argument('douyin_view_cost_price', type=int)
                args = self.parse.parse_args()
                douyinve = DouyinViewExport()
                douyinve.resource_table_id = resource_id
                douyinve.export_tag = args.get('export_tag')
                douyinve.introduction = args.get('introduction')
                douyinve.douyin_home_page = args.get('douyin_home_page')
                douyinve.export_city = args.get('export_city')
                douyinve.cooperation_case = args.get('cooperation_case')
                douyinve.better_sell_goods = args.get('better_sell_goods')
                douyinve.douyin_export_classification = args.get('douyin_export_classification')
                douyinve.cooperation_mode = args.get('cooperation_mode')
                douyinve.offer_less = args.get('offer_less')
                douyinve.offer_more = args.get('offer_more')
                douyinve.star_offer = args.get('star_offer')
                douyinve.douyin_view_cost_price = args.get('douyin_view_cost_price')
                try:
                    douyinve.save(douyinve)
                    # dyve_query = DouyinViewExport.query.offset((args['page']) - 1 * args['limit']).limit(
                    #     args['limit']).all()
                    # totle = DouyinViewExport.query.count()
                    # dyve_dict = DouyinViewExport.queryToDict(dyve_query)
                    # return jsonify({'code': 20000, 'data': {'total': totle, 'items': dyve_dict}})
                except Exception as e:
                    print('DouyinViewExport save except', e)
            elif flag == '快手直播':  # 快手直播
                self.parse.add_argument('avg_online_num', type=float)
                self.parse.add_argument('sell_classification', type=str)
                self.parse.add_argument('commission_less', type=int)
                self.parse.add_argument('commission_more', type=int)
                self.parse.add_argument('attributes', type=str)
                self.parse.add_argument('better_sell_goods', type=str)
                self.parse.add_argument('kuaishou_offer', type=int)
                self.parse.add_argument('kuaishou_cost_price', type=int)
                self.parse.add_argument('remarks', type=str)
                args = self.parse.parse_args()
                ksl = KuaiShouLive()
                ksl.resource_table_id = resource_id
                ksl.avg_online_num = args.get('avg_online_num')
                ksl.sell_classification = args.get('sell_classification')
                ksl.commission_less = args.get('commission_less')
                ksl.commission_more = args.get('commission_more')
                ksl.attributes = args.get('attributes')
                ksl.kuaishou_offer = args.get('kuaishou_offer')
                ksl.kuaishou_cost_price = args.get('kuaishou_cost_price')
                ksl.remarks = args.get('remarks')
                try:
                    ksl.save(ksl)

                    ksl_query = KuaiShouLive.query.offset((args['page']) - 1 * args['limit']).limit(
                        args['limit']).all()
                    # totle = KuaiShouLive.query.count()
                    # ksl_dict = KuaiShouLive.queryToDict(ksl_query)
                    # return jsonify({'code': 20000, 'data': {'total': totle, 'items': ksl_dict}})
                except Exception as e:
                    print('KuaiShouLive save except', e)
            elif flag == '小红书图文链接':  # 小红书图文链接
                print('抖音专场直播 args1')
                self.parse.add_argument('dianzan', type=int)
                self.parse.add_argument('redbook_link', type=str)
                self.parse.add_argument('export_city', type=str)
                self.parse.add_argument('export_tag', type=str)
                self.parse.add_argument('brand_partner', type=bool)
                self.parse.add_argument('redbook_cost_price', type=int)
                args = self.parse.parse_args()
                print('抖音专场直播 args1',args)
                rdltl = RedbookImageTextLink()
                rdltl.resource_table_id = resource_id
                rdltl.dianzan = args.get('dianzan')
                rdltl.redbook_link = args.get('redbook_link')
                rdltl.export_city = args.get('export_city')
                rdltl.export_tag = args.get('export_tag')
                rdltl.brand_partner = args.get('brand_partner')
                rdltl.redbook_cost_price = args.get('redbook_cost_price')
                try:
                    rdltl.save(rdltl)
                    rdltl_query = RedbookImageTextLink.query.offset((args['page']) - 1 * args['limit']).limit(
                        args['limit']).all()
                    # totle = RedbookImageTextLink.query.count()
                    # rdltl_dict = RedbookImageTextLink.queryToDict(rdltl_query)
                    # return jsonify({'code': 20000, 'data': {'total': totle, 'items': rdltl_dict}})
                except Exception as e:
                    print('RedbookImageTextLink save except', e)
            elif flag == '淘宝直播':  # 淘宝直播
                print('抖音专场直播 args')
                self.parse.add_argument('avg_viewing_num', type=float)
                self.parse.add_argument('main_category', type=str)
                self.parse.add_argument('introduction', type=str)
                self.parse.add_argument('taobao_offer', type=int)
                self.parse.add_argument('taobao_cost_price', type=int)
                args = self.parse.parse_args()
                print('抖音专场直播 args1', args)
                tbl = TaobaoLive()
                tbl.resource_table_id = resource_id
                tbl.avg_viewing_num = args.get('avg_viewing_num')
                tbl.main_category = args.get('main_category')
                tbl.introduction = args.get('introduction')
                tbl.taobao_offer = args.get('taobao_offer')
                tbl.taobao_cost_price = args.get('taobao_cost_price')
                try:
                    tbl.save(tbl)
                    # tbl_query = TaobaoLive.query.offset((args['page']) - 1 * args['limit']).limit(
                    #     args['limit']).all()
                    # totle = TaobaoLive.query.count()
                    # tbl_dict = TaobaoLive.queryToDict(tbl_query)
                    # return jsonify({'code': 20000, 'data': {'total': totle, 'items': tbl_dict}})
                except Exception as e:
                    print('TaobaoLive save except', e)
        except Exception as e:
            db.session.rollback()
            print('TotalResource post except', e)
            return e

    def update(self, department_data):
        """
        更新部门

        :param department_data: 部门信息
        :return: 已经更新的部门id
        """
        if 'id' not in department_data:
            raise AttributeError('必须包含Id')
        return self._post(
            '/department/update',
            department_data,
            result_processor=lambda x: x['id']
        )

    def xx_get(self,select_type,page,limit):
        """
        资源

        :param page: 页码
        :param limit: 标签数量
        :return:
        """
        res = self._get('/platformresource/list/resourcetable',params={'select_type':select_type,'page':page,'limit':limit})
        print('xx__gte',res)

        return res


    @classmethod
    def get_datas(self, request, model=None):

        headers = request.headers
        content_type = headers.get('Content-Type')
        print(content_type)
        if request.method == "GET":
            return request.args
        if content_type == 'application/x-www-form-urlencoded':
            print("1")
            return request.form
        if content_type.startswith('application/json'):
            print("2")
            return request.get_json()

        content_type_list = str(content_type).split(';')
        if len(content_type_list) > 0:
            if content_type_list[0] == 'multipart/form-data':
                print("3")
                return request.form

#     def list(self, department_id, offset=0, size=100, order='custom', lang='zh_CN'):
#         """
#         获取部门成员（详情）
#
#         :param department_id: 获取的部门id
#         :param offset: 偏移量
#         :param size: 表分页大小，最大100
#         :param order: 排序规则
#                       entry_asc     代表按照进入部门的时间升序
#                       entry_desc    代表按照进入部门的时间降序
#                       modify_asc    代表按照部门信息修改时间升序
#                       modify_desc   代表按照部门信息修改时间降序
#                       custom        代表用户定义排序
#         :param lang: 通讯录语言(默认zh_CN另外支持en_US)
#         :return:
#         """
#         return self._get(
#             '/user/list',
#             {
#                 'department_id': department_id,
#                 'offset': offset,
#                 'size': size,
#                 'order': order,
#                 'lang': lang
#             }
#         )
#
# class DingTalkBaseAPI(object):
#     API_BASE_URL = None
#     def __init__(self, client=None):
#         self._client = client
#
#     def _get(self, url, params=None, **kwargs):
#         if self.API_BASE_URL:
#             kwargs['api_base_url'] = self.API_BASE_URL
#         return self._client.get(url, params, **kwargs)
#
# class BaseClient(object):
#
#     _http = requests.Session()
#
#     API_BASE_URL = 'https://oapi.dingtalk.com/'
#     def get(self, uri, params=None, **kwargs):
#         """
#         get 接口请求
#
#         :param uri: 请求url
#         :param params: get 参数（dict 格式）
#         """
#         if params is not None:
#             kwargs['params'] = params
#         return self.request('GET', uri, **kwargs)
#
#     def request(self, method, uri, **kwargs):
#         method, uri_with_access_token, kwargs = self._handle_pre_request(method, uri, kwargs)
#
#         def _handle_pre_request(self, method, uri, kwargs):
#             return method, uri, kwargs
#         try:
#             return self._request(method, uri_with_access_token, **kwargs)
#         except DingTalkClientException as e:
#             return self._handle_request_except(e, self.request, method, uri, **kwargs)



ResourceApi.add_resource(TotalResource, '/resourcetable')





# -*- coding: utf-8 -*-
from sqlalchemy import String, Integer, Date, Column, ForeignKey, BigInteger, Float

from aishowapp.ext import db
from aishowapp.models import BaseDef

# 达人列表
class KolList(db.Model, BaseDef):

    __tablename__ = 'kol_list'

    userid = Column(BigInteger, primary_key=True)
    nickname = Column(String(255))
    follower_num = Column(Integer)
    tag = Column(String(255))
    video_count = Column(Integer)
    avg_interaction_15 = Column(Integer)
    age = Column(Integer)
    sex = Column(String(255))

    # 服务器
    Fans_description = Column(String(255))

    #需要增加
    view_link = Column(String(255))
    # 自己

    #页面访问量
    pv = Column(Integer)
    #独立访客
    uv = Column(Integer)
    #
    # tv = Column(String(255))
    #热度
    heat = Column(Integer)

    # word_cloud = Column(String(255))

    c_time = Column(Date)

    ebusines_id = db.relationship("Ebusines", backref='kolList')
    fansattribute_id = db.relationship("FansAttribute", backref='kolList')
    shoppingcart_id = db.relationship("ShoppingCart", backref='kolList')
    phoneageinfo_id = db.relationship("PhoneAgeInfo", backref='kolList')
    region_id = db.relationship("Region", backref='kolList')

# 电商表
class Ebusines(db.Model, BaseDef):
    __tablename__ = 'ebusiness'

    userid = Column(ForeignKey('kol_list.userid', ondelete='CASCADE', onupdate='CASCADE'), primary_key=True)
    datatime = Column(Date)
    goods_category = Column(String(255))
    Commodity_classification = Column(String(255))
    Video_likes = Column(Integer)
    Video_count = Column(Integer)
    Total_browsing = Column(Integer)
    Sales_volume = Column(Integer)
    sell_price = Column(Integer)
    Cargo_link = Column(String(255))

# 粉丝属性
class FansAttribute(db.Model, BaseDef):
    __tablename__ = 'fans_attributes'

    userid = Column(ForeignKey('kol_list.userid', ondelete='CASCADE', onupdate='CASCADE'), primary_key=True)
    avg_play = Column(Float(asdecimal=True))
    avg_like = Column(Float(asdecimal=True))
    avg_comment = Column(Float(asdecimal=True))
    total_video_count = Column(Integer)
    video_count_15 = Column(Integer)
    video_count_30 = Column(Integer)
    total_video_avg_interaction = Column(Float(asdecimal=True))
    avg_interaction_15 = Column(Float(asdecimal=True))
    avg_interaction_30 = Column(Float(asdecimal=True))

# 商品购物车
class ShoppingCart(db.Model, BaseDef):
    __tablename__ = 'shopping_cart'

    userid = Column(ForeignKey('kol_list.userid', ondelete='CASCADE', onupdate='CASCADE'), primary_key=True)
    LE10 = Column(BigInteger)
    GT10LE50 = Column(BigInteger)
    GT50LE100 = Column(BigInteger)
    GT100LE150 = Column(BigInteger)
    GT150LE200 = Column(BigInteger)
    GT200LE300 = Column(BigInteger)
    GT300LE400 = Column(BigInteger)
    GT400LE500 = Column(BigInteger)
    GT500 = Column(BigInteger)

# 手机年龄信息
class PhoneAgeInfo(db.Model, BaseDef):
    __tablename__ = 'phone_age_info'

    userid = Column(ForeignKey('kol_list.userid', ondelete='CASCADE', onupdate='CASCADE'), primary_key=True)
    iPhone = Column(BigInteger)
    OPPO = Column(BigInteger)
    vivo = Column(BigInteger)
    others = Column(BigInteger)
    huawei = Column(BigInteger)
    xiaomi = Column(BigInteger)
    less_18 = Column(BigInteger)
    age_18_25 = Column(BigInteger)
    age_26_32 = Column(BigInteger)
    age_33_39 = Column(BigInteger)
    age_40_46 = Column(BigInteger)
    greater_46 = Column(BigInteger)
    male = Column(BigInteger)
    female = Column(BigInteger)
    one_city = Column(Float(asdecimal=True))
    two_city = Column(Float(asdecimal=True))
    three_city = Column(Float(asdecimal=True))

# 地域
class Region(db.Model, BaseDef):
    __tablename__ = 'region'

    userid = Column(ForeignKey('kol_list.userid', ondelete='CASCADE', onupdate='CASCADE'), primary_key=True)
    qingh = Column(BigInteger)
    heil = Column(BigInteger)
    shangd = Column(BigInteger)
    sic = Column(BigInteger)
    jiangs = Column(BigInteger)
    guiz = Column(BigInteger)
    xingj = Column(BigInteger)
    fuj = Column(BigInteger)
    zhej = Column(BigInteger)
    hub = Column(BigInteger)
    tianj = Column(BigInteger)
    jiangx = Column(BigInteger)
    xiz = Column(BigInteger)
    heilj = Column(BigInteger)
    guangd = Column(BigInteger)
    yunn = Column(BigInteger)
    beij = Column(BigInteger)
    taiw = Column(BigInteger)
    aom = Column(BigInteger)
    guangx = Column(BigInteger)
    shan3x = Column(BigInteger)
    gans = Column(BigInteger)
    heb = Column(BigInteger)
    ningx = Column(BigInteger)
    chongq = Column(BigInteger)
    jil = Column(BigInteger)
    hun = Column(BigInteger)
    neimg = Column(BigInteger)
    anh = Column(BigInteger)
    xiangg = Column(BigInteger)
    shangh = Column(BigInteger)
    shan1x = Column(BigInteger)
    hain = Column(BigInteger)
    liaon = Column(BigInteger)


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import six

# Six 就是来解决这个烦恼的，这是一个专门用来兼容 Python 2 和 Python 3 的模块，
    # 它解决了诸如 urllib 的部分方法不兼容， str 和 bytes 类型不兼容等“知名”问题。
    # import six
    # six.PY2 #返回一个表示当前运行环境是否为python2的boolean值
    # six.PY3 #返回一个表示当前运行环境是否为python3的boolean值
    # six.integer_types # 在python2中，存在 int 和 long 两种整数类型；在python3中，仅存在一种类型int
    # six.string_types # 在python2中，使用的为basestring；在python3中，使用的为str
    # six.text_type # 在python2中，使用的文本字符的类型为unicode；在python3中使用的文本字符的类型为str
    # six.binary_type # 在python2中，使用的字节序列的类型为str；在python3中使用的字节序列的类型为bytes
    # from dingtalk.core.utils import to_binary, to_text


from aishow_model_app.core.utils import to_binary, to_text


class DingTalkException(Exception):

    def __init__(self, errcode, errmsg):
        """
        :param errcode: Error code
        :param errmsg: Error message
        """
        self.errcode = errcode
        self.errmsg = errmsg

    def __str__(self):
        _repr = 'Error code: {code}, message: {msg}'.format(
            code=self.errcode,
            msg=self.errmsg
        )

        if six.PY2:
            return to_binary(_repr)
        else:
            return to_text(_repr)

    def __repr__(self):
        _repr = '{klass}({code}, {msg})'.format(
            klass=self.__class__.__name__,
            code=self.errcode,
            msg=self.errmsg
        )
        if six.PY2:
            return to_binary(_repr)
        else:
            return to_text(_repr)


class DingTalkClientException(DingTalkException):
    """WeChat API client exception class"""
    def __init__(self, errcode, errmsg, client=None,
                 request=None, response=None):
        super(DingTalkClientException, self).__init__(errcode, errmsg)
        self.client = client
        self.request = request
        self.response = response


class InvalidSignatureException(DingTalkException):
    """Invalid signature exception class"""

    def __init__(self, errcode=-40001, errmsg='Invalid signature'):
        super(InvalidSignatureException, self).__init__(errcode, errmsg)


class InvalidCorpIdOrSuiteKeyException(DingTalkException):
    """Invalid app_id exception class"""

    def __init__(self, errcode=-40005, errmsg='Invalid CorpIdOrSuiteKey'):
        super(InvalidCorpIdOrSuiteKeyException, self).__init__(errcode, errmsg)



# -*- coding: utf-8 -*-
import sys

from flask import Flask
from aishow_model_app.apis import init_api
from aishow_model_app.ext import init_ext
from config import configs


sys.path.append(sys.modules['__main__'])

def create_app(env):
    app = Flask(__name__)
    app.config.from_object(configs[env])
    configs[env].init_app(app)
    init_ext(app)
    init_api(app)

    # app.wsgi_app = UserMiddleware(app.wsgi_app)

    return app





# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from aishow_model_app.apis.base import BaseAPI


class User(BaseAPI):

    def auth_scopes(self):
        """
        获取CorpSecret授权范围

        :return:
        """
        return self._get('/auth/scopes')

    def get_org_user_count(self, only_active):
        """
        获取企业员工人数

        :param only_active: 是否包含未激活钉钉的人员数量
        :return: 企业员工数量
        """
        return self._get(
            '/user/get_org_user_count',
            {'onlyActive': 0 if only_active else 1},
            result_processor=lambda x: x['count']
        )

    def getuserinfo(self, code):
        """
        通过CODE换取用户身份

        :param code: requestAuthCode接口中获取的CODE
        :return:
        """
        return self._get(
            '/user/getuserinfo',
            {'code': code}
        )

    def get(self, userid, lang='zh_CN'):
        """
        获取成员详情

        :param userid: 员工在企业内的UserID，企业用来唯一标识用户的字段
        :param lang: 通讯录语言(默认zh_CN，未来会支持en_US)
        :return:
        """
        return self._get(
            '/user/get',
            {'userid': userid, 'lang': lang}
        )

    def create(self, user_data):
        """
        创建成员

        :param user_data: 用户信息
        :return: userid
        """
        return self._post(
            '/user/create',
            user_data,
            result_processor=lambda x: x['userid']

        )

    def update(self, user_data):
        """
        更新成员

        :param user_data: 用户信息
        :return:
        """
        return self._post(
            '/user/update',
            user_data
        )

    def delete(self, userid):
        """
        删除成员

        :param userid: 员工在企业内的UserID，企业用来唯一标识用户的字段
        :return:
        """
        return self._get(
            '/user/delete',
            {'userid': userid}
        )

    def batchdelete(self, user_ids):
        """
        批量删除成员

        :param user_ids: 员工UserID列表。列表长度在1到20之间
        :return:
        """
        return self._post(
            '/user/delete',
            {'useridlist': list(user_ids)}
        )

    def simple_list(self, department_id, offset=0, size=100, order='custom', lang='zh_CN'):
        """
        获取部门成员

        :param department_id: 获取的部门id
        :param offset: 偏移量
        :param size: 表分页大小，最大100
        :param order: 排序规则
                      entry_asc     代表按照进入部门的时间升序
                      entry_desc    代表按照进入部门的时间降序
                      modify_asc    代表按照部门信息修改时间升序
                      modify_desc   代表按照部门信息修改时间降序
                      custom        代表用户定义排序
        :param lang: 通讯录语言(默认zh_CN另外支持en_US)
        :return:
        """
        return self._get(
            '/user/simplelist',
            {
                'department_id': department_id,
                'offset': offset,
                'size': size,
                'order': order,
                'lang': lang
            }
        )

    def list(self, department_id, offset=0, size=100, order='custom', lang='zh_CN'):
        """
        获取部门成员（详情）

        :param department_id: 获取的部门id
        :param offset: 偏移量
        :param size: 表分页大小，最大100
        :param order: 排序规则
                      entry_asc     代表按照进入部门的时间升序
                      entry_desc    代表按照进入部门的时间降序
                      modify_asc    代表按照部门信息修改时间升序
                      modify_desc   代表按照部门信息修改时间降序
                      custom        代表用户定义排序
        :param lang: 通讯录语言(默认zh_CN另外支持en_US)
        :return:
        """
        return self._get(
            '/user/list',
            {
                'department_id': department_id,
                'offset': offset,
                'size': size,
                'order': order,
                'lang': lang
            }
        )

    def get_admin(self):
        """
        获取管理员列表

        :return: sys_level	管理员角色 1:主管理员,2:子管理员
        """
        return self._get(
            '/user/get_admin',
            result_processor=lambda x: x['admin_list']
        )

    def can_access_microapp(self, app_id, user_id):
        """
        获取管理员的微应用管理权限

        :param app_id: 微应用id
        :param user_id: 员工唯一标识ID
        :return: 是否能管理该微应用
        """
        return self._get(
            '/user/can_access_microapp',
            {'appId': app_id, 'userId': user_id},
            result_processor=lambda x: x['canAccess']
        )

    def get_userid_by_unionid(self, unionid):
        """
        根据unionid获取成员的userid

        :param unionid: 用户在当前钉钉开放平台账号范围内的唯一标识
        :return:
        """
        return self._get(
            '/user/getUseridByUnionid',
            {'unionid': unionid}
        )

    def get_dept_member(self, dept_id):
        """
        获取部门用户userid列表

        :param dept_id: 用户在当前钉钉开放平台账号范围内的唯一标识
        :return 部门userid列表:
        """
        return self._get(
            '/user/getDeptMember',
            {'deptId': dept_id},
            result_processor=lambda x: x['userIds']
        )

    def listbypage(self, department_id, offset=0, size=100, order='custom', lang='zh_CN'):
        """
        获取部门用户

        :param department_id: 获取的部门id
        :param offset: 偏移量
        :param size: 表分页大小，最大100
        :param order: 排序规则
                      entry_asc     代表按照进入部门的时间升序
                      entry_desc    代表按照进入部门的时间降序
                      modify_asc    代表按照部门信息修改时间升序
                      modify_desc   代表按照部门信息修改时间降序
                      custom        代表用户定义排序
        :param lang: 通讯录语言(默认zh_CN另外支持en_US)
        :return:
        """
        return self._get(
            '/user/list',
            {
                'department_id': department_id,
                'offset': offset,
                'size': size,
                'order': order,
                'lang': lang
            }
        )

    def get_admin_scope(self, userid):
        """
        查询管理员通讯录权限范围

        :param userid: 用户id
        """
        return self._top_request(
            "dingtalk.oapi.user.get_admin_scope",
            {"userid": userid},
            result_processor=lambda x: x['dept_ids']
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import hashlib
import json
import random
import string

import six


class ObjectDict(dict):

    """Makes a dictionary behave like an object, with attribute-style access.
    """

    def __getattr__(self, key):
        if key in self:
            return self[key]
        return None

    def __setattr__(self, key, value):
        self[key] = value


class DingTalkSigner(object):

    """ signer  符号; 征候; 签名; 署名;"""
    """DingTalk data signer"""


    def __init__(self, delimiter=b''):

        """
        :param delimiter:分隔符
        """
        self._data = []
        self._delimiter = to_binary(delimiter)

    def add_data(self, *args):
        """Add data to signer"""
        for data in args:
            self._data.append(to_binary(data))

    @property
    def signature(self):
        """Get data signature"""
        self._data.sort()
        str_to_sign = self._delimiter.join(self._data)
        return hashlib.sha1(str_to_sign).hexdigest()


def to_text(value, encoding='utf-8'):

    """Convert value to unicode, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return ''
    if isinstance(value, six.text_type):
        return value
    if isinstance(value, six.binary_type):
        return value.decode(encoding)
    return six.text_type(value)


def to_binary(value, encoding='utf-8'):
    """Convert value to binary string, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return b''
    if isinstance(value, six.binary_type):
        return value
    if isinstance(value, six.text_type):
        return value.encode(encoding)
    return to_text(value).encode(encoding)


def random_string(length=16):
    """
    随机生成字符串
    :param length:
    :return:
    """
    """ascii_letters方法的作用是生成全部字母, 包括a - z, A - Z2.digits方法的作用是生成数组, 包括0 - 9"""
    rule = string.ascii_letters + string.digits
    rand_list = random.sample(rule, length)

    return ''.join(rand_list)


def byte2int(c):

    """
    ord()函数是chr()函数（对于8位的ASCII字符串）或unichr()函数（对于Unicode对象）的配对函数，它以一个字符（长度为1的字符串）
    作为参数，返回对应的ASCII数值，或者Unicode数值，如果所给的Unicode字符超出了你的Python定义范围，则会引发一个TypeError
    的异常。
    语法
    ord()方法的语法:
        ord(c)参数
        c - - 字符。
        返回值
            返回值是对应的十进制整数。
    """
    if six.PY2:
        return ord(c)
    return c


def json_loads(s, object_hook=ObjectDict, **kwargs):
    """
    object_hook参数是可选的，它会将（loads的)返回结果字典替换为你所指定的类型
    :param s:
    :param object_hook:
    :param kwargs:
    :return:
    """
    return json.loads(s, object_hook=object_hook, **kwargs)



# -*- coding: utf-8 -*-
from flask_caching import Cache
from flask_moment import Moment
from flask_cors import CORS
from flask_sqlalchemy import SQLAlchemy
from flask_session import Session
from flask_bootstrap import Bootstrap
from flask_migrate import Migrate
from flask_login import LoginManager
from flask_pagedown import PageDown

db=SQLAlchemy()
migrate=Migrate()
moment = Moment()
session=Session()
# bootstarp = Bootstrap()
cache=Cache(config={'CACHE_TYPE':'simple'})
# pagedown = PageDown()
login_manager = LoginManager()
login_manager.session_protection = 'strong'
login_manager.login_view = 'auth.login'

def init_ext(app):
    CORS(app, supports_credentials=True)
    db.init_app(app=app)
    session.init_app(app)
    moment.init_app(app)
    migrate.init_app(app, db)
    # bootstarp.init_app(app)
    cache.init_app(app)
    login_manager.init_app(app)
    # pagedown.init_app(app)



# -*- coding: utf-8 -*-

from aishowapp.ext import db
from aishowapp.models import BaseDef

# 达人
class Mcn(db.Model,BaseDef):
    """
    达人
    """
    __tablename__ = 'mcn'

    id = db.Column(db.Integer, primary_key=True,autoincrement=True)
    mcn_id = db.Column(db.String(64))
    broker = db.Column(db.Integer, db.ForeignKey('users.id'))  #中间人
    nickname = db.Column(db.String(64))
    name = db.Column(db.String(64))
    # platform = db.Column(db.Enum(
    #     '快手', '抖音', '淘宝', '西瓜', 'bilibili', '美拍', '秒拍', '新浪微博'
    # ),  nullable=False,default=None)
    platform_id = db.Column(db.Integer, db.ForeignKey('platform.id'))  #平台
    way = db.Column(db.String(64))     #方式
    sex = db.Column(db.Boolean, default=False)
    fans = db.Column(db.Integer)    #粉丝
    viewers = db.Column(db.Integer)   #观看者
    Online = db.Column(db.Float) #在线时长
    region = db.Column(db.String(64)) #地区
    commission = db.Column(db.Float) # 佣金
    attributes = db.Column(db.String(64))   #属性
    price = db.Column(db.Float)     # 价格
    invoicing = db.Column(db.Boolean, default=False)  #是否开票
    live_url = db.Column(db.String(128))  #直播地址
    brand = db.Column(db.String(64))      #品牌
    data = db.Column(db.String(64))       #时间
    gent = db.Column(db.Integer, db.ForeignKey('users.id'))  # 经纪人


# 平台
class PlatForm(db.Model,BaseDef):
    """
    平台
    """
    __tablename__ = 'platform'
    id = db.Column(db.Integer, primary_key=True,autoincrement=True)
    name = db.Column(db.String(64))
    Mcns = db.relationship("Mcn", backref='platform')


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from enum import Enum

#解释Enum
# Python——枚举（enum）
#     使用普通类直接实现枚举
# 　　在Python中，枚举和我们在对象中定义的类变量时一样的，每一个类变量就是一个枚举项，访问枚举项的方式为：类名加上类变量，像下面这样：
#
# class color():
#     YELLOW = 1
#     RED = 2
#     GREEN = 3
#     PINK = 4
#
# # 访问枚举项
# print(color.YELLOW)  # 1
# 　　虽然这样是可以解决问题的，但是并不严谨，也不怎么安全，比如：
# 　　1、枚举类中，不应该存在key相同的枚举项（类变量）
# 　　2、不允许在类外直接修改枚举项的值
#
# class color():
#     YELLOW = 1
#     YELLOW = 3  # 注意这里又将YELLOW赋值为3，会覆盖前面的1
#     RED = 2
#     GREEN = 3
#     PINK = 4
#
# # 访问枚举项
# print(color.YELLOW)  # 3
# # 但是可以在外部修改定义的枚举项的值，这是不应该发生的
# color.YELLOW = 99
# print(color.YELLOW)  # 99
# 解决方案：使用enum模块
# 　　enum模块是系统内置模块，可以直接使用import导入，但是在导入的时候，不建议使用import
# enum将enum模块中的所有数据都导入，一般使用的最多的就是enum模块中的Enum、IntEnum、unique这几项
#
# # 导入枚举类
# from enum import Enum
#
# # 继承枚举类
# class color(Enum):
#     YELLOW = 1
#     BEOWN = 1
#     # 注意BROWN的值和YELLOW的值相同，这是允许的，此时的BROWN相当于YELLOW的别名
#     RED = 2
#     GREEN = 3
#     PINK = 4
#
# class color2(Enum):
#     YELLOW = 1
#     RED = 2
#     GREEN = 3
#     PINK = 4
#
# 　　使用自己定义的枚举类：
# print(color.YELLOW)  # color.YELLOW
# print(type(color.YELLOW))  # <enum 'color'>
# print(color.YELLOW.value)  # 1
# print(type(color.YELLOW.value))  # <class 'int'>
# print(color.YELLOW == 1)  # False
# print(color.YELLOW.value == 1)  # True
# print(color.YELLOW == color.YELLOW)  # True
# print(color.YELLOW == color2.YELLOW)  # False
# print(color.YELLOW is color2.YELLOW)  # False
# print(color.YELLOW is color.YELLOW)  # True
# print(color(1))  # color.YELLOW
# print(type(color(1)))  # <enum 'color'>
# 　　注意事项如下：
# 　　1、枚举类不能用来实例化对象
# 　　2、访问枚举类中的某一项，直接使用类名访问加上要访问的项即可，比如color.YELLOW
# 　　3、枚举类里面定义的Key = Value，在类外部不能修改Value值，也就是说下面这个做法是错误的color.YELLOW = 2  # Wrong, can't reassign member
# 　　4、枚举项可以用来比较，使用 ==，或者is
# 　　5、导入Enum之后，一个枚举类中的Key和Value，Key不能相同，Value可以相，但是Value相同的各项Key都会当做别名，
# 　　6、如果要枚举类中的Value只能是整型数字，那么，可以导入IntEnum，然后继承IntEnum即可，注意，此时，如果value为字符串的数字，也不会报错：from enum import IntEnum
# 　　7、如果要枚举类中的key也不能相同，那么在导入Enum的同时，需要导入unique函数from enum import Enum, unique


class SuitePushType(Enum):
    """套件相关回调枚举"""
    CHECK_URL = "check_url"  # 校验url
    CHANGE_AUTH = "change_auth"  # 授权变更
    SUITE_TICKET = "suite_ticket"  # 套件ticket     ticket 票; 券; 车票; 戏票; 入场券; 奖券;标签
    TMP_AUTH_CODE = "tmp_auth_code"  # 临时授权码
    SUITE_RELIEVE = "suite_relieve"  # 解除授权
    CHECK_CREATE_SUITE_URL = "check_create_suite_url"  # 校验创建套件时候的url
    CHECK_UPDATE_SUITE_URL = "check_update_suite_url"  # 校验更改套件时候的url
    CHECK_SUITE_LICENSE_CODE = "check_suite_license_code"  # 校验序列号
    MARKET_BUY = "market_buy"  # 用户购买下单
    ORG_MICRO_APP_STOP = "org_micro_app_stop"  # 企业逻辑停用微应用
    ORG_MICRO_APP_REMOVE = "org_micro_app_remove"  # 企业物理删除微应用
    ORG_MICRO_APP_RESTORE = "org_micro_app_restore"  # 企业逻辑启用微应用



# -*- coding: utf-8 -*-
from sqlalchemy import String, Integer, Column, ForeignKey, BigInteger, Float, Boolean

from aishow_model_app.ext import db
from aishow_model_app.models import BaseDef
from config import USE



# 资源总表
class ResourceTable(db.Model,BaseDef):

    # 资源总表

        __tablename__ = 'resource_table'
        __bind_key__ = USE
        id = Column(BigInteger, primary_key=True)
        kol_name = Column(String(128),nullable=False,unique=True)     #达人昵称
        kol_id = Column(String(128),nullable=False,unique=True)        #达人平台ID
        platform = Column(String(128),nullable=False)       #平台
        avatar = Column(String(128),nullable=True)       #头像
        type_datil = Column(String(128),nullable=False)    #合作模式（专场直播，单链直播，直播，短视频，图文链接）
        company = Column(String(128),nullable=True)         #所属公司        BD不可见
        contact_name = Column(String(128),nullable=True)     #联系人         BD不可见
        contact_phone = Column(String(128),nullable=True)     #电话          BD不可见
        fans = Column(Float,nullable=False)                    #粉丝数(万）
        total_sell_money = Column(String(255),nullable=True)   #与麒腾累计带货合作金额
        cooperation_times = Column(Integer,nullable=True)                #合作次数
        # cooperation_times1 = Column(Integer,nullable=True)                #合作次数


        redbook_image_text_links = db.relationship('RedbookImageTextLink', backref='resource_tables')
        douyin_view_exports = db.relationship("DouyinViewExport", backref='resource_tables')
        douyin_special_lives = db.relationship("DouyinSpecialLive", backref='resource_tables')
        douyin_single_chain_lives = db.relationship("DouyinSingleChainLive", backref='resource_tables')
        taobao_lives = db.relationship("TaobaoLive", backref='resource_tables')
        kuai_show_lives = db.relationship("KuaiShouLive", backref='resource_tables')

#抖音达人分类
class DouyinExportClassification(db.Model, BaseDef):

    __tablename__ = 'douyin_export_classification'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    classification_description = Column(String(64),unique=True,)  # 分类描述


#小红书图文链接
class RedbookImageTextLink(db.Model, BaseDef):

    __tablename__ = 'redbook_image_text_link'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True)
    resource_table_id = Column(BigInteger, ForeignKey('resource_table.id',ondelete='CASCADE', onupdate='CASCADE'))  # 资源表ID
    dianzan = Column(Integer, default=0)  # 点赞量级(万）
    redbook_link = Column(String(255), nullable=False, unique=True)  # 链接
    export_city = Column(String(255), nullable=True)  # 达人所在城市
    export_tag = Column(String(128), default=0)  # 达人标签
    brand_partner = Column(Boolean, default=False)  # 是否品牌合作人
    redbook_cost_price = Column(Integer, default=0)  # 成本价   BD不可见


# 麒腾小红书3月报价
class QitengRedbookPrice(db.Model,BaseDef):

    __tablename__ = 'qiteng_redbook_price'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True,autoincrement=True)
    fans_less = Column(Integer, default=0)     #粉丝量级区间最低(万）
    fans_more = Column(Integer, default=0)        #粉丝量级区间最高(万）
    offer_less = Column(Integer, default=0)  # 报价区间最低值      630
    offer_more = Column(Integer, default=0)  # 报价区间最高值      900
    cost_price = Column(Integer, default=0)     #成本价
    remarks = Column(String(255),nullable=True)         #备注
    brand_partner= Column(Boolean, default=False)     #是否品牌合作人
    redbook_image_text_link_id = Column(BigInteger, ForeignKey('redbook_image_text_link.id', ondelete='CASCADE',onupdate='CASCADE'))  # 资源表ID
    redbook_image_text_linkid = db.relationship('RedbookImageTextLink', backref='redbook_image_text_links',uselist=False)


#抖音短视频达人表
class DouyinViewExport(db.Model, BaseDef):

    __tablename__ = 'douyin_view_export'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True,autoincrement=True)
    resource_table_id = Column(BigInteger, ForeignKey('resource_table.id',ondelete='CASCADE', onupdate='CASCADE'))  # 资源表ID
    douyin_export_classification_id = Column(BigInteger,ForeignKey('douyin_export_classification.id',ondelete='CASCADE', onupdate='CASCADE'))  # 类目属性
    export_tag = Column(String(255),nullable=True)  # 达人标签
    introduction = Column(String(255),nullable=True)  # 简介 可能是多个tag标签
    douyin_home_page = Column(String(255),nullable=False,unique=True)  # 抖音主页url
    export_city = Column(String(255),nullable=True)  # 达人所在城市
    cooperation_case = Column(String(255),nullable=True)  # 达人合作过的品牌
    better_sell_goods = Column(String(255),nullable=True)  # 可能是空值，达人销售过较好的商品
    douyin_export_classification = Column(String(255))  # 抖音达人分类
    cooperation_mode = Column(String(255),nullable=False)  # 合作模式
    offer_less = Column(Integer, default=0)  # 报价区间最低值      630
    offer_more = Column(Integer, default=0)  # 报价区间最高值      900
    star_offer = Column(Integer, default=0)  # 星图参考报价最高   3000
    douyin_view_cost_price = Column(Integer, default=0)  # 成本价   BD不可见


#麒腾抖音短视频3月报价
class QitengDouyinViewPrice(db.Model, BaseDef):

    __tablename__ = 'qiteng_douyin_view_price'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    fans_less = Column(Float, default=0)   # 粉丝量级区间最低(万） 1w
    fans_more = Column(Float, default=0)   # 粉丝量级区间最高(万） 2w
    offer_less = Column(Integer, default=0)  # 报价区间最低值      630
    offer_more = Column(Integer, default=0)  # 报价区间最高值      900
    star_offer_less = Column(Integer, default=0)  # 星图参考报价最低   1700
    star_offer_more = Column(Integer, default=0)  # 星图参考报价最低   3000
    estimated_exposure = Column(Float, default=0)  # 预估曝光     100w
    remarks = Column(String(255),nullable=False)  # 备注       30条起作这个价格或下方粉丝量有数量可抵
    douyin_view_export_id = Column(BigInteger, ForeignKey('douyin_view_export.id', ondelete='CASCADE',
                                                               onupdate='CASCADE'))  # 资源表ID
    douyin_view_exportid = db.relationship('DouyinViewExport', backref='douyin_view_exports',uselist=False)


#抖音专场直播
class DouyinSpecialLive(db.Model, BaseDef):

    __tablename__ = 'douyin_special_live'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    resource_table_id = Column(BigInteger, ForeignKey('resource_table.id',ondelete='CASCADE', onupdate='CASCADE'))  # 资源表ID
    export_tag = Column(String(128),nullable=False)  # 达人标签
    special_offer = Column(Integer, default=0)  # 专场报价区间最低值      630
    export_city = Column(String(64),nullable=True)  # 达人所在城市
    cooperation_case = Column(String(255),nullable=True)  # 达人合作过的品牌
    douyin_special_cost_price = Column(Integer, default=0)  # 成本价   BD不可见


#抖音单链直播
class DouyinSingleChainLive(db.Model, BaseDef):

    __tablename__ = 'douyin_single_chain_live'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    resource_table_id = Column(BigInteger, ForeignKey('resource_table.id',ondelete='CASCADE', onupdate='CASCADE'))  # 资源表ID
    douyin_export_classification = Column(String(255),nullable=False)  # 达人适合的类目 抖音大类，可能多个
    Single_chain_offer = Column(Integer, default=0)  # 单链接报价区间      630
    introduction = Column(String(255),nullable=True)  # 简介
    selection_requirements = Column(String(255),nullable=False)  # 选品要求   颜值高，成分好的美妆产品
    live_time = Column(String(64))  # 直播时间,每天\每周N
    remarks = Column(String(255), nullable=True)  # 备注
    douyin_single_cost_price = Column(Integer, default=0)  # 成本价   BD不可见


#淘宝直播
class TaobaoLive(db.Model, BaseDef):

    __tablename__ = 'taobao_live'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    resource_table_id = Column(BigInteger, ForeignKey('resource_table.id',ondelete='CASCADE', onupdate='CASCADE'))  # 资源表ID
    avg_viewing_num = Column(Float,default=0)  # 场均观看/小时
    main_category = db.Column(db.String(64),nullable=False)  # 主营类目
    introduction = Column(String(255), nullable=True)  # 简介
    taobao_offer = Column(Integer,default=0)  #最高
    taobao_cost_price = Column(Integer, default=0)  # 成本价   BD不可见


#麒腾淘宝KOL直播报价
class QitengTaobaoExportLiveOffer(db.Model, BaseDef):

    __tablename__ = 'qiteng_taobao_export_live_offer'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    hierarchy = Column(String(64),nullable=False)  # 层级 优质、腰部、头部、超头部
    avg_viewing_num_less = Column(Float,default=0)  # 场均观看量级低区间最低万
    avg_viewing_num_more = Column(Float,default=0)  # 场均观看量级低 区间最高万
    offer_less = Column(Integer,default=0)  #最低报价
    offer_more = Column(Integer,default=0)  #最高报价
    remarks = Column(String(255), nullable=True)  # 备注
    cost_price = Column(Integer,default=0)  # 成本价   BD不可见
    taobao_live_id = Column(BigInteger, ForeignKey('taobao_live.id', ondelete='CASCADE',
                                                          onupdate='CASCADE'))  # 资源表ID
    taobao_liveid = db.relationship('TaobaoLive', backref='taobao_lives',uselist=False)


# 快手直播
class KuaiShouLive(db.Model, BaseDef):

    __tablename__ = 'kuai_show_live'
    __bind_key__ = USE

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    resource_table_id = Column(BigInteger, ForeignKey('resource_table.id',ondelete='CASCADE', onupdate='CASCADE'))  # 资源表ID
    avg_online_num = Column(Float, default=0)  #单位：平均在线人数(万)
    sell_classification = Column(String(255),nullable=False)  # 可售卖类目
    commission_less = Column(Integer, default=0)   # 佣金最低范围最低，例如10%
    commission_more = Column(Integer, default=0)   # 佣金最高范围最高，例如10%
    attributes = db.Column(db.String(64),nullable=False)  # 属性 带货、刷榜
    kuaishou_offer = db.Column(Integer, default=0)  # 快手报价（单链接）
    kuaishou_cost_price = Column(Integer, default=0)  # 快手成本价   BD不可见
    remarks = Column(String(255),nullable=True)  # 备注


#

    # douyin_view_exports = db.relationship("DouyinViewExport", backref='douyin_export_classification')



#! /usr/bin/env python
# encoding: utf-8

"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
# To use a consistent encoding
from codecs import open
from os import path
import sys

# ???????
# import ssl
# try:
#     ssl._create_default_https_context = ssl._create_unverified_context
# except Exception:
#     pass

here = path.abspath(path.dirname(__file__))

# ???????
# class PyTest(TestCommand):
#     user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]
#
#     def initialize_options(self):
#         TestCommand.initialize_options(self)
#         self.pytest_args = []
#
#     def finalize_options(self):
#         TestCommand.finalize_options(self)
#         self.test_args = []
#         self.test_suite = True
#
#     def run_tests(self):
#         import pytest
#         errno = pytest.main(self.pytest_args)
#         sys.exit(errno)

# ?????
# cmdclass = {}
# cmdclass['test'] = PyTest

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

with open('requirements.txt') as f:
    requirements = [l for l in f.read().splitlines() if l]

setup(
    name='zly-resource-module',                                    #项目名
    version='1.0.1',                                        #版本
    keywords='dingding, ding, dtalk, dingtalk, SDK',        #关键字
    description='DingTalk SDK for Python',                  #简短描述
    long_description=long_description,                      #详细描述
    # url='https://github.com/007gzs/dingtalk-sdk',         #网址
    # author='007gzs',
    # author_email='007gzs@sina.com',
    license='MIT',                                           #许可证
    classifiers=[                                            #分类器
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    # packages=[
    #     'aishow_model_app',
    #     'aishow_model_app.apis',
    #     'aishow_model_app.models',
    #     'aishow_model_app.storage',
    #     'aishow_model_app.model',
    #     'aishow_model_app.templates',
    #     'aishow_model_app.docs',
    #     'aishow_model_app.tests',
    # ],
    py_modules=["config","manage"],
    install_requires=requirements,
    zip_safe=False,
    include_package_data=True,
    python_requires='>=3.6',
    # tests_require=[
    #     'pytest',
    #     'redis',
    #     'pymemcache',
    # ],
    # cmdclass=cmdclass,
    # extras_require={
    #     'cryptography': ['cryptography'],
    #     'pycrypto': ['pycrypto'],
    # },
)



# -*- coding: utf-8 -*-
import os
import redis

basedir = os.path.abspath(os.path.dirname(__file__))

# USE = 'wr'
USE = 'zly'


API_BASE_URL='http://192.168.0.58:8001/'


class Config:

    SECRET_KEY = "FDHUFHSIFHSOIAFJSIOAJDShuhdh242424"
    # 数据库
    SQLALCHEMY_TRACK_MODIFICATIONS = True

# redis
#     REDIS_HOST = '127.0.0.1'
#     REDIS_PORT = 6379
#     CACHE_REDIS_PASSWORD='qt@demo123'
 # flask_session的配置
 #    SESSION_TYPE = "redis"
 #    SESSION_REDIS = redis.StrictRedis(host=REDIS_HOST, port=REDIS_PORT)
 #    SESSION_USE_SIGNER = True  # 对cookie中的session_id进行隐藏处理
 #    PERMANENT_SESSION_LIFETIME = 86400  # session数据的过期时间

    # CACHE_TYPE: 'redis'
    # CACHE_REDIS_HOST: '127.0.0.1'
    # CACHE_REDIS_PORT: 6379
    # CACHE_REDIS_DB: ''
    # CACHE_REDIS_PASSWORD: ''

    # CACHE_REDIS_PASSWORD: 'qt@demo123'  #服务器


class DevelopmentConfig(Config):

    #开发环境
    SQLALCHEMY_DATABASE_URI = "mysql+pymysql://root:123@127.0.0.1:3306/star_library?charset=utf8"
    # SQLALCHEMY_DATABASE_URI = "mysql+pymysql://root:Qt@123456!@172.16.23.2:3306/zlytest?charset=utf8"
    # 本地链接
    # SQLALCHEMY_DATABASE_URI = "mysql+pymysql://root:123@127.0.0.1/star_library?charset=utf8"
    SQLALCHEMY_BINDS = {
        # 线上代码部署
        # 'wr': 'mysql+pymysql://root:Qt@123456!@172.16.23.2:3306/mcn?charset=utf8&',
        # 'zly': 'mysql+pymysql://root:Qt@123456!@172.16.23.2:3306/zlytest?charset=utf8&'

        #本地链接
        'wr': 'mysql+pymysql://root:root123@192.168.1.159:3306/bpm?charset=utf8&',
        'zly': 'mysql+pymysql://root:123@127.0.0.1:3306/star_library?charset=utf8&',

    }
    DEBUG = True

    @classmethod
    def init_app(cls, app):
        pass


class TestingConfig(Config):
    #测试环境
    SQLALCHEMY_DATABASE_URI = "mysql+pymysql://root:Qt@123456!@172.16.23.2/star_library?charset=utf8"
    # SQLALCHEMY_DATABASE_URI = "mysql+pymysql://root:123@127.0.0.1/star_dev?charset=utf8"
    TESTING = True

    @classmethod
    def init_app(cls, app):
        pass


class ProductionConfig(Config):

    @classmethod
    def init_app(cls, app):
        pass


configs = {
    'development': DevelopmentConfig,
    'testing': TestingConfig,
    'production': ProductionConfig,
    'default': DevelopmentConfig
}


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import inspect
import json
import logging
import requests
import six
from six.moves.urllib.parse import urljoin

"""
    # >>>from urllib.parse import urljoin
    # >>> urljoin("http://www.chachabei.com/folder/currentpage.html", "anotherpage.html")
    # 'http://www.chachabei.com/folder/anotherpage.html'
    # >>> urljoin("http://www.chachabei.com/folder/currentpage.html", "/anotherpage.html")
    # 'http://www.chachabei.com/anotherpage.html'
    # >>> urljoin("http://www.chachabei.com/folder/currentpage.html", "folder2/anotherpage.html")
    # 'http://www.chachabei.com/folder/folder2/anotherpage.html'
    # >>> urljoin("http://www.chachabei.com/folder/currentpage.html", "/folder2/anotherpage.html")
    # 'http://www.chachabei.com/folder2/anotherpage.html'
    # >>> urljoin("http://www.chachabei.com/abc/folder/currentpage.html", "/folder2/anotherpage.html")
    # 'http://www.chachabei.com/folder2/anotherpage.html'
    # >>> urljoin("http://www.chachabei.com/abc/folder/currentpage.html", "../anotherpage.html")
    # 'http://www.chachabei.com/abc/anotherpage.html'
"""

from aishow_model_app.apis.base import BaseAPI
from aishow_model_app.core.exceptions import DingTalkClientException
from aishow_model_app.core.utils import json_loads
from aishow_model_app.storage.memorystorage import MemoryStorage



logger = logging.getLogger(__name__)

#判断是否是DingTalkBaseAPI的实列对象
def _is_api_endpoint(obj):
    return isinstance(obj, BaseAPI)


class BaseClient(object):

    _http = requests.Session()

    API_BASE_URL = 'http://192.168.0.58:8001/'

    def __new__(cls, *args, **kwargs):
        self = super(BaseClient, cls).__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_api_endpoint)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(self)
            setattr(self, name, api)
        return self

    def __init__(self, storage=None, timeout=None, auto_retry=True):
        self.storage = storage or MemoryStorage()
        self.timeout = timeout
        self.auto_retry = auto_retry  #自动重播


    #对request进行封装
    def _request(self, method, url_or_endpoint, **kwargs):
        # 将基础路径和试图函数拼接成完整的访问路劲
        if not url_or_endpoint.startswith(('http://', 'https://')):
            api_base_url = kwargs.pop('api_base_url', self.API_BASE_URL)
            url = urljoin(api_base_url, url_or_endpoint)
        else:
            url = url_or_endpoint

        #判断是否有params参数，
        if 'params' not in kwargs:
            kwargs['params'] = {}
        # 判断是否有data参数，
        if isinstance(kwargs.get('data', ''), dict):
            body = json.dumps(kwargs['data'], ensure_ascii=False)
            body = body.encode('utf-8')
            kwargs['data'] = body
            # 判断是否有headers参数，
            if 'headers' not in kwargs:
                kwargs['headers'] = {}
            kwargs['headers']['Content-Type'] = 'application/json'

        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        result_processor = kwargs.pop('result_processor', None)    #processor:处理器 ???????????
        top_response_key = kwargs.pop('top_response_key', None)     # ?????????
        #发送requests.Session()的request()请求
        res = self._http.request(
            method=method,
            url=url,
            **kwargs
        )
        #异常处理返回的数剧是否正常
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            logger.error("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【异常信息】：%s",
                         url, kwargs.get('params', ''), kwargs.get('data', ''), reqe)

            #返回自定义的DingTalkClientException异常处理类
            raise DingTalkClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response
            )

        #处理返回的结果
        result = self._handle_result(
            res, method, url, result_processor, top_response_key, **kwargs
        )

        logger.debug("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【响应数据】：%s",
                     url, kwargs.get('params', ''), kwargs.get('data', ''), result)
        return result

    def _decode_result(self, res):
        try:
            result = json_loads(res.content.decode('utf-8', 'ignore'), strict=False)
            # 1、ValueError: Invalid control characterat: line 1 column 8363(char8362)使用json.loads(json_data)时，出现：
            #     ValueError: Invalid control character at: line 1column 8363(char8362)
            # 出现错误的原因是字符串中包含了回车符（\r）或者换行符（\n）
            #     解决方法：
            #     (1)对这些字符转义：json_data = json_data.replace('\r', '\\r').replace('\n', '\\n')
            #     (2)使用关键字strict:json.loads(json_data, strict=False)strict默认是True, 它将严格控制内部字符串，将其设置为False, 便可以允许你\n \r

        except (TypeError, ValueError):
            # Return origin response object if we can not decode it as JSON
            logger.debug('Can not decode response as JSON', exc_info=True)
                # logger.debug('Can not decode response as JSON', exc_info=True)
                # exc_info： 其值为布尔值，如果该参数的值设置为True，则会将异常异常信息添加到日志消息中。如果没有异常信息则添加None到日志信息中。
                # stack_info： 其值也为布尔值，默认值为False。如果该参数的值设置为True，栈信息将会被添加到日志信息中。
                # extra： 这是一个字典（dict）参数，它可以用来自定义消息格式中所包含的字段，但是它的key不能与logging模块定义的字段冲突。
            return res
        return result

    def _handle_result(self, res, method=None, url=None, result_processor=None, top_response_key=None, **kwargs):
        """
       :param res:
       :param method:
       :param url:
       :param result_processor:  结果处理;
       :param top_response_key:
       :param kwargs:
       :return:
       """
        if not isinstance(res, dict):
            # Dirty hack around asyncio based AsyncWeChatClient
            result = self._decode_result(res)
        else:
            result = res

        if not isinstance(result, dict):
            return result

        if top_response_key:
            if 'error_response' in result:
                error_response = result['error_response']
                logger.error("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【错误信息】：%s",
                             url, kwargs.get('params', ''), kwargs.get('data', ''), result)
                raise DingTalkClientException(
                    error_response.get('code', -1),
                    error_response.get('sub_msg', error_response.get('msg', '')),
                    client=self,
                    request=res.request,
                    response=res
                )
            top_result = result
            if top_response_key in top_result:
                top_result = result[top_response_key]
                if 'result' in top_result:
                    top_result = top_result['result']
                    if isinstance(top_result, six.string_types):
                        try:
                            top_result = json_loads(top_result)
                        except Exception:
                            pass
            if isinstance(top_result, dict):
                if ('success' in top_result and not top_result['success']) or (
                        'is_success' in top_result and not top_result['is_success']):
                    logger.error("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【错误信息】：%s",
                                 url, kwargs.get('params', ''), kwargs.get('data', ''), result)
                    raise DingTalkClientException(
                        top_result.get('ding_open_errcode', -1),
                        top_result.get('error_msg', ''),
                        client=self,
                        request=res.request,
                        response=res
                    )
            result = top_result
        if not isinstance(result, dict):
            return result
        if 'errcode' in result:
            result['errcode'] = int(result['errcode'])

        if 'errcode' in result and result['errcode'] != 0:
            errcode = result['errcode']
            errmsg = result.get('errmsg', errcode)

            logger.error("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【错误信息】：%s",
                         url, kwargs.get('params', ''), kwargs.get('data', ''), result)
            raise DingTalkClientException(
                errcode,
                errmsg,
                client=self,
                request=res.request,
                response=res
            )

        return result if not result_processor else result_processor(result)

    def _handle_pre_request(self, method, uri, kwargs):
        return method, uri, kwargs

    def _handle_pre_top_request(self, params, uri):
        if not uri.startswith(('http://', 'https://')):
            uri = urljoin('https://eco.taobao.com', uri)
        return params, uri

    def _handle_request_except(self, e, func, *args, **kwargs):
        raise e

    def request(self, method, uri, **kwargs):
        method, uri_with_access_token, kwargs = self._handle_pre_request(method, uri, kwargs)
        try:
            return self._request(method, uri_with_access_token, **kwargs)
        except DingTalkClientException as e:
            return self._handle_request_except(e, self.request, method, uri, **kwargs)

    def top_request(self, method, params=None, format_='json', v='2.0',
                    simplify='false', partner_id=None, url=None, **kwargs):
        """
        top 接口请求

        :param method: API接口名称。
        :param params: 请求参数 （dict 格式）
        :param format_: 响应格式（默认json，如果使用xml，需要自己对返回结果解析）
        :param v: API协议版本，可选值：2.0。
        :param simplify: 是否采用精简JSON返回格式
        :param partner_id: 合作伙伴身份标识。
        :param url: 请求url，默认为 https://eco.taobao.com/router/rest
        """
        from datetime import datetime

        reqparams = {}
        if params is not None:
            for key, value in params.items():
                reqparams[key] = value if not isinstance(value, (dict, list, tuple)) else json.dumps(value)
        reqparams['method'] = method
        reqparams['timestamp'] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        reqparams['format'] = format_
        reqparams['v'] = v

        if format_ == 'json':
            reqparams['simplify'] = simplify
        if partner_id:
            reqparams['partner_id'] = partner_id
        base_url = url or '/router/rest'

        reqparams, base_url = self._handle_pre_top_request(reqparams, base_url)

        if not base_url.startswith(('http://', 'https://')):
            base_url = urljoin(self.API_BASE_URL, base_url)
        response_key = method.replace('.', '_') + "_response"
        try:
            return self._request('POST', base_url, params=reqparams, top_response_key=response_key, **kwargs)
        except DingTalkClientException as e:
            return self._handle_request_except(e, self.request,
                                               method, format_, v, simplify, partner_id, url, params, **kwargs)

    def get(self, uri, params=None, **kwargs):
        """
        get 接口请求

        :param uri: 请求url
        :param params: get 参数（dict 格式）
        """
        if params is not None:
            kwargs['params'] = params
        # return self._client.get(url=none, kwargs['params']={'select_type':select_type,'page':page,'limit':limit})
        return self.request('GET', uri, **kwargs)

    def post(self, uri, data=None, params=None, **kwargs):
        """
        post 接口请求

        :param uri: 请求url
        :param data: post 数据（dict 格式会自动转换为json）
        :param params: post接口中url问号后参数（dict 格式）
        """
        if data is not None:
            kwargs['data'] = data
        if params is not None:
            kwargs['params'] = params
        return self.request('POST', uri, **kwargs)
